#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sasl/sasl.h>
#include <glib.h>
#include <syslog.h>

#define MAX_LINE 1024

int main(void) {
    char buf[MAX_LINE];
    sasl_conn_t *conn = NULL;

    // Open syslog for this process
    openlog("squid_sasl_helper", LOG_PID | LOG_NDELAY, LOG_AUTH);

    // Initialize SASL
    if (sasl_server_init(NULL, "squid") != SASL_OK) {
        syslog(LOG_ERR, "Failed to initialize SASL");
        fprintf(stderr, "ERR\n");
        return 1;
    }

    // Create a SASL server connection context
    int result = sasl_server_new("squid", NULL, NULL, NULL, NULL, NULL, 0, &conn);
    if (result != SASL_OK) {
        syslog(LOG_ERR, "sasl_server_new failed: %s", sasl_errstring(result, NULL, NULL));
        fprintf(stderr, "ERR\n");
        sasl_done();
        closelog();
        return 1;
    }

    // Read input from Squid
    while (fgets(buf, sizeof(buf), stdin)) {
        // Strip newline
        buf[strcspn(buf, "\n")] = 0;  // Remove the newline character from the end of the input

        // Split username and password using Glib
        gchar **tokens = g_strsplit(buf, " ", 2);
        gchar *username = tokens[0];
        gchar *password = tokens[1];

        // Ensure both username and password are present
        if (!username || !password) {
            syslog(LOG_WARNING, "Missing username or password");
            printf("ERR\n");
            g_strfreev(tokens);
            fflush(stdout);
            continue;
        }

        // Use g_strstrip to remove leading and trailing spaces (including newlines)
        g_strstrip(username);
        g_strstrip(password);

        // Attempt SASL authentication
        result = sasl_checkpass(conn, username, strlen(username), password, strlen(password));

        if (result == SASL_OK) {
            // Successful authentication
            syslog(LOG_INFO, "Authentication succeeded for user '%s'", username);
            printf("OK\n");
        } else {
            // Authentication failed
            const char *errstr = sasl_errstring(result, NULL, NULL);
            syslog(LOG_WARNING, "SASL auth failed for user '%s': %s", username, errstr);
            printf("ERR\n");
        }

        g_strfreev(tokens);  // Free memory allocated by g_strsplit
        fflush(stdout);
    }

    // Cleanup
    sasl_dispose(&conn);
    sasl_done();
    closelog();
    return 0;
}

